@@ -1,11 +1,10 @@
-from functools import partial
-from multiprocessing import dummy
 import os
 import shutil
 import yaml
 import tal
+from tal import __version__ as tal_version
 from tal.io.capture_data import NLOSCaptureData
-from tal.io.enums import FileFormat, GridFormat, HFormat
+from tal.enums import FileFormat, GridFormat, HFormat
 from tal.util import fdent, write_img, tonemap_ldr
 from tal.render.mitsuba2_transient_nlos import (
     get_material_keys, get_materials,
@@ -18,7 +17,7 @@ import numpy as np
 from tqdm import tqdm
 
 
-def get_scene_xml(config, quiet=False):
+def get_scene_xml(config, random_seed=0, quiet=False):
     def s(value):
         if isinstance(value, bool):
             return str(value).lower()
@@ -110,6 +109,7 @@ def get_scene_xml(config, quiet=False):
             </transform>
             <sampler type="independent">
                 <integer name="sample_count" value="512"/>
+                <integer name="seed" value="{random_seed}"/>
             </sampler>
             <film type="hdrfilm">
                 <integer name="width" value="512"/>
@@ -151,6 +151,7 @@ def get_scene_xml(config, quiet=False):
         <sensor type="nloscapturemeter">
             <sampler type="independent">
                 <integer name="sample_count" value="{v('sample_count')}"/>
+                <integer name="seed" value="{random_seed}"/>
             </sampler>
 
             <emitter type="projector">
@@ -166,6 +167,7 @@ def get_scene_xml(config, quiet=False):
             <film type="streakhdrfilm" name="streakfilm">
                 <integer name="width" value="{v('sensor_width')}"/>
                 <integer name="height" value="{v('sensor_height')}"/>
+                <string name="pixel_format" value="rgb"/>
                 <string name="component_format" value="float32"/>
 
                 <integer name="num_bins" value="{v('num_bins')}"/>
@@ -244,13 +246,14 @@ def get_scene_xml(config, quiet=False):
 
             shapes_steady.append(shapify(shape_contents_steady, filename))
             shapes_nlos.append(shapify(shape_contents_nlos, filename))
-        elif g('mesh')['type'] == 'rectangle':
+        elif g('mesh')['type'] == 'rectangle' or g('mesh')['type'] == 'sphere':
             def shapify(content):
                 return fdent('''\
                 {shape_name}
-                <shape type="rectangle">
+                <shape type="{shape_type}">
                     {content}
-                </shape>''', shape_name=shape_name, content=content)
+                </shape>''', shape_name=shape_name, content=content,
+                             shape_type=g('mesh')['type'])
 
             shapes_steady.append(shapify(shape_contents_steady))
             shapes_nlos.append(shapify(shape_contents_nlos))
@@ -318,8 +321,10 @@ def render_nlos_scene(config_path, args):
             f'Invalid YAML format in TAL config file: {exc}') from exc
     scene_config = {**scene_defaults, **scene_config}
 
-    mitsuba_set_variant(scene_config['mitsuba_variant'])
-    steady_xml, nlos_xml = get_scene_xml(scene_config, quiet=args.quiet)
+    if not args.dry_run:
+        mitsuba_set_variant(scene_config['mitsuba_variant'])
+    steady_xml, nlos_xml = get_scene_xml(
+        scene_config, random_seed=args.seed, quiet=args.quiet)
 
     try:
         root_dir = os.path.join(
@@ -354,6 +359,9 @@ def render_nlos_scene(config_path, args):
         sensor_height = scene_config['sensor_height']
         laser_width = scene_config['laser_width']
         laser_height = scene_config['laser_height']
+        if scan_type == 'single':
+            laser_width = 1
+            laser_height = 1
 
         relay_wall = next(filter(
             lambda g: g['name'] == scene_config['relay_wall'],
@@ -368,9 +376,9 @@ def render_nlos_scene(config_path, args):
 
         def get_grid_xyz(nx, ny, relay_wall_scale):
             p = relay_wall_scale
-            xg = np.stack((np.linspace(-p, p, num=2*nx + 1)[1::2],)*ny, axis=0)
-            yg = np.stack((np.linspace(-p, p, num=2*ny + 1)[1::2],)*nx, axis=1)
-            assert xg.shape[0] == yg.shape[0] == ny and xg.shape[1] == yg.shape[1] == nx, \
+            xg = np.stack((np.linspace(-p, p, num=2*nx + 1)[1::2],)*ny, axis=1)
+            yg = np.stack((np.linspace(-p, p, num=2*ny + 1)[1::2],)*nx, axis=0)
+            assert xg.shape[0] == yg.shape[0] == nx and xg.shape[1] == yg.shape[1] == ny, \
                 'Incorrect shapes'
             return np.stack([xg, yg, np.zeros((nx, ny))], axis=-1).astype(np.float32)
 
@@ -392,8 +400,6 @@ def render_nlos_scene(config_path, args):
             np.array([0, 0, 1]), laser_width, laser_height)
 
         if scan_type == 'single':
-            laser_width = 1.0
-            laser_height = 1.0
             laser_lookat_x = \
                 scene_config['laser_lookat_x'] or (sensor_width - 1) / 2
             laser_lookat_y = \
@@ -458,7 +464,7 @@ def render_nlos_scene(config_path, args):
                     log_dir,
                     f'{experiment_name}_L[{laser_lookat_x}][{laser_lookat_y}].log'), 'w')
             run_mitsuba(nlos_scene_xml, exr_dir, defines,
-                        f'Laser {i} of {len(laser_lookats)}', logfile, args)
+                        f'Laser {i + 1} of {len(laser_lookats)}', logfile, args)
             if args.do_logging and not args.dry_run:
                 logfile.close()
 
@@ -492,6 +498,7 @@ def render_nlos_scene(config_path, args):
         capture_data.t_accounts_first_and_last_bounces = \
             scene_config['account_first_and_last_bounces']
         capture_data.scene_info = {
+            'tal_version': tal_version,
             'config': scene_config,
             'args': vars(args),
         }
